--------------------------------------------------------------------------------
--  File     : default_chapter.lua
--  Author(s): Pavel Kislyak
--
--  Summary  :    .
--
--  Copyright  2013-2015 Hullabu Inc.
--------------------------------------------------------------------------------
dofile("_config.lua")
dofile("_helpers.lua")

--------------------------------------------------------------------------------
function OnStart()
	--    Kontagent
	kontagent:Event("start", 0, "case_progress", GetCaseNumber(), "")

	--      
	game_info.case.stars = 0

	--    
	game_info.case_progress[chapter:GetName()] = 0

	ui.GamePanel:SetActivePersonageWidget("alex_rhianna")
	StartLoadGlobalRooms("OnCustomEndLoadRooms")
end

--------------------------------------------------------------------------------
function StartLoadGlobalRooms(callbackFinishFuncName)
	chapter:AddRoomToLoadTask(chapter_settings.QuestItemsRoom)
	chapter:AddRoomToLoadTask(chapter_settings.DashboardRoom)
	chapter:StartLoadingRooms(callbackFinishFuncName)
end

--------------------------------------------------------------------------------
function OnGameRestored()
	ui:CloseDialog("DialogOk")
	ui:CloseDialog("DialogYesNo")

	local cur_progress = game_info.case_progress[chapter:GetName()]
	if cur_progress == 100 and game_center:IsAuthenticated() then
		game_center:ReportAchievement(chapter:GetName() .. "_completed")
	end

	if not room then
		JumpDashboard()
	end
end

--------------------------------------------------------------------------------
function OnUpdate(elapsedTime)
	local currOsTime = os.time()
	local add_bonus_energy = true

	if game_info.energy >= new_player_settings.Energy then
		game_info.energy_up_timer = 0
		game_info.energy_update_time = currOsTime
		top_panel:HideEnergyTimer()
	else
		--     
		local dt = os.difftime(currOsTime, game_info.energy_update_time)
		top_panel:UpdateEnergyTimer(game_settings.AddEnergyPerTime - (game_info.energy_up_timer + dt))

		if game_info.energy_up_timer + dt >= game_settings.AddEnergyPerTime then
			--         1  
			game_info.energy = game_info.energy + 1
			game_info.energy_up_timer = game_info.energy_up_timer + dt - game_settings.AddEnergyPerTime

			--  
			if game_info.energy == new_player_settings.Energy then
				game_info.energy_up_timer = 0
				top_panel:HideEnergyTimer()
			end

			--     
			game_info.energy_update_time = currOsTime

			--     
			add_bonus_energy = false
		end
	end

	if add_bonus_energy and game_info.energy_bonus > 0 then
		game_info.energy = game_info.energy + 1
		game_info.energy_bonus = game_info.energy_bonus - 1
	end
end

--------------------------------------------------------------------------------
--      
--------------------------------------------------------------------------------
function OnStartLoadRooms()
	CloseClose(ui.ChapterTasks)
	CloseClose(ui.DashboardPanel)
	CloseClose(ui.TopPanel)
	CloseClose(ui.GamePanel)
	CloseClose(ui.StartRoom)

	ui:ShowDialog("LoadingScreen")
end

function OnEndLoadRooms()
	ui:CloseDialog("LoadingScreen")
end

function OnCustomEndLoadRooms()
	JumpDashboard()
end

--------------------------------------------------------------------------------
--   OnDialogClose_VideoPlayer  _helpers.lua       
--------------------------------------------------------------------------------
function OnDialogClose_VideoPlayer(dialog)
	if dialog:GetUserTag() == "show dashboard" then
		chapter:JumpRoom(chapter_settings.DashboardRoom)
	end
end

--------------------------------------------------------------------------------
--  Options
--------------------------------------------------------------------------------
function OnDialogShow_Options(dialog)
	chapter:ReturnCursorObjectInPlace()
	chapter:SetScenePause(true)
end

function OnDialogClose_Options(dialog)
	chapter:SetScenePause(false)
end

--------------------------------------------------------------------------------
--  YesNo
--------------------------------------------------------------------------------
function OnDialogShow_DialogYesNo(dialog)
	--         
	if dialog:GetUserTag() == "room_exit" then
		chapter:SetScenePause(true)
	end
end

function OnDialogClose_DialogYesNo(dialog)
	if dialog:GetUserTag() == "room_exit" then
		chapter:SetScenePause(false)
	end
end

--------------------------------------------------------------------------------
--  CaseComplete
--------------------------------------------------------------------------------
function OnDialogShow_CaseComplete(dialog)
	if game_info.energy < new_player_settings.Energy then
		kontagent:Event("case_complete", new_player_settings.Energy - game_info.energy, "economy", "energy", "source")
		game_info.energy = new_player_settings.Energy
	end
end

--------------------------------------------------------------------------------
--      
--------------------------------------------------------------------------------
function OnJumpCloseup(name)
	ui:CloseDialog("PersComment")
end

function OnJumpRoom(name)
	ui:CloseDialog("PersComment")
	CloseClose(ui.HintMG)
	CloseClose(ui.BuyHint)
end

--------------------------------------------------------------------------------
-- FACEBOOK
--------------------------------------------------------------------------------
function OnFacebookLogin(state)
	if state == true then
		if game_info.facebook.login_reward == false then
			--   
			LogInfo("*FB* Got login reward in chapter!")
			TransferRewardToUser(rewards.facebook_login, "fb_connect")
			game_info.facebook.login_reward = true
		end
		if gstore.action_after_facebook_login == 1 then
			gstore.action_after_facebook_login = 2
		end
		ui.FBNeedLogin:Close()
		ui.DashboardPanel:SetWidgetVisible("FBConnect", false)
	else --[[if room and room:GetName() == chapter_settings.DashboardRoom then]]
		ui.DashboardPanel:SetWidgetVisible("FBConnect", true)
		if gstore.action_after_facebook_login ~= -1 then
			ui.AlertDialog:Show(0, "FBNeedLogin:ALERT_CAPTION", "FBNeedLogin:ALERT_TEXT", "", "_Common:btn_ok", "", "", 0.4, 0)
		end
	end
	gstore.action_after_facebook_login = 0
end

--    Facebook
function OnFacebookResponceReceived(name)
	if name == "GetRequest_MeAppRequests" then
		if facebook:GetGiftsCount() > 0 and ui.FBGetGift:IsWaitResponce() == true then
			game_info.facebook.getgift_show = true
		end
	elseif name == "GetRequest_UserFriends" then
		ui.FBGift:ReceiveRequests()
	end
end
