--------------------------------------------------------------------------------
--  File     : default_dashboard.lua
--  Author(s): Pavel Kislyak, Pavel Demchuk
--
--  Summary  :   Dashboard.
--
--  Copyright  2013-2015 Hullabu Inc.
--------------------------------------------------------------------------------
dofile("_config.lua")
dofile("_helpers.lua")

--------------------------------------------------------------------------------
function OnAfterLoadInit()
	afterLoadInit = true
end

--------------------------------------------------------------------------------
function OnEntry()
	-- GUI
	game_panel:Close()
	top_panel:ShowDashboardMode()
	ui.DashboardPanel:Show()
	ui.DashboardPanel:SetWidgetEnabled("Tasks_btn", true)
	ui.DashboardPanel:SetWidgetEnabled("Friends_btn", false)

	--  ,    ,  
	music:Restart("dashboard", true)

	--  
	ambient:Play("dashboard")

	--     -
	--[[for k,v in pairs(tasks_settings) do
		NewTask(k)
	end]]

	checkForLowEnergy = true
	checkForAvailableTasks = true

	--      
	SafeCallFunction(OnCustomEntry)

	if primary_tasks:IsEmpty() then
		CalcCollectibles()
		checkForAvailableTasks = afterLoadInit
		afterLoadInit = false
	end

	UpdateDailyReward()
	ProcessFacebookDialogs()

	--  case01     Playhaven
	local caseNum = tonumber(GetCaseNumber())
	if type(caseNum) == 'number' and caseNum > 0 then
		playhaven:ShowAd("case_" .. caseNum .. "_scene_selection_screen")
	end
end

--------------------------------------------------------------------------------
function OnExit()
	ambient:Stop()
	--   Playhaven
	playhaven:PreloadAdContent("case_" .. GetCaseNumber() .. "_scene_selection_screen")
end

--------------------------------------------------------------------------------
function OnUpdate(elapsedTime)
	--     
	if not music:IsPlaying() then
		music:Play("dashboard", true)
	end

	-- Attempts show Facebook dialogs
	if facebook:IsLogged() and not ui:IsModal() and facebook_settings.Enabled == 1 then
		if game_info.facebook.getgift_show == true then
			game_info.facebook.getgift_show = false
			game_info.facebook.getgift_time = os.time()
			ui.FBGetGift:ReceiveRequests()
			if ui.FBGetGift:IsShown() then
				kontagent:Event("opened", 0, "social", "message", "")
			end
		end
	end

	--   
	if not ui:IsModal() and not ui.LevelUp:IsShown() then
		if game_info.xp_points >= GetLevelXpPoints(game_info.xp_level + 1) then
			local show_share = IsSharingEnabled()
			ui.LevelUp:SetWidgetVisible("Share",  show_share)
			ui.LevelUp:SetWidgetVisible("Ok", not show_share)
			ui.LevelUp:Show()
		end
	end

	--    
	ShowProgress()

	--   ,    CaseComplete (      LevelUp!)
	ShowCaseComplete()

	--   
	if not ui:IsModal() and dashboard:CheckNewUnlockedRoom() then
		ui.RoomUnlocked:Show(dashboard:GetNewUnlockedRoomCard())
	end

	--   ,       
	if not ui:IsModal() and checkForLowEnergy then
		if IsEnergyLow() then
			ui.BuyEnergy:SetUserTag("out_of_energy")
			ui.BuyEnergy:Show()
		end
		checkForLowEnergy = false
	end

	--     
	if not ui:IsModal() and not ui.ChapterTasks:IsShown() then
		--          (     Dashboard)
		if primary_tasks:IsNewFlag() or (checkForAvailableTasks and (primary_tasks:CheckToReadyTasks() or primary_tasks:IsEmpty())) then
			ui.ChapterTasks:Show()
			checkForAvailableTasks = false
		end
	end
end

--------------------------------------------------------------------------------
function OnFocusGain()
	ProcessFacebookDialogs()
end

--------------------------------------------------------------------------------
function OnClickOnRoomCard(roomCard)
	ui.ChapterTasks:Close()

	--    VisitButton
	--tutorial:HideActive()

	if roomCard:IsLocked() then
		ui.UnlockRoom:Show(roomCard)
	else
		ui.StartRoom:Show(roomCard)
		--     Play
		if tutorial:ShowWidget("StartRoom/Play", "PlayButton", 1, -1, 590, -50, 7, true) then
			game_info:CreateTextEvent("tutorial_step_06")
			kontagent:Event("tap_play", 0, "tutorial", "step_06", "")
		end
	end
end

--------------------------------------------------------------------------------
--  RoomUnlocked
--------------------------------------------------------------------------------
function OnDialogClose_RoomUnlocked(dialog)
	OnClickOnRoomCard(dialog:GetRoomCard())
end

--------------------------------------------------------------------------------
--   StartRoom
--------------------------------------------------------------------------------
function OnDialogClick_StartRoom(dialog, widget)
	if widget == "Play" then
		if game_info.energy >= GetPlayEnergyCost() then
			--    
			game_info.energy = game_info.energy - GetPlayEnergyCost()
			kontagent:Event("scene", -GetPlayEnergyCost(), "economy", "energy", "sink")

			--   
			music:Play(chapter_settings.Music, true)

			--     ,    
			local roomName = dialog:GetRoomName()
			SafeCallFunction(OnStartRoomLoading, roomName)

			JumpStoryRoom(roomName)
			dialog:OnStartRoom()
		else
			--   
			ui.BuyEnergy:SetUserTag("out_of_energy")
			ui.BuyEnergy:Show()
		end
	end
end

--------------------------------------------------------------------------------
--     
--------------------------------------------------------------------------------
function OnStartTask_JumpRoom(taskId, roomName)
	if dashboard:IsRoomCardExists(roomName) then
		--     
		OnClickOnRoomCard(dashboard:GetRoomCard(roomName))
	elseif game_info.energy >= GetPlayEnergyCost() then
		--    
		game_info.energy = game_info.energy - GetPlayEnergyCost()
		kontagent:Event("task", -GetPlayEnergyCost(), "economy", "energy", "sink")

		--   
		music:Play(chapter_settings.Music, true)

		--     ,    
		SafeCallFunction(OnStartRoomLoading, roomName)

		--       
		JumpStoryRoom(roomName)
	else
		--   
		ui.BuyEnergy:SetUserTag("out_of_energy")
		ui.BuyEnergy:Show()
	end
end

--------------------------------------------------------------------------------
--  DashboardPanel
--------------------------------------------------------------------------------
function OnDialogClick_DashboardPanel(dialog, widget)
	if widget == "Tasks_btn" then
		if ui.ChapterTasks:IsShown() then
			ui.ChapterTasks:Close()
		else
			ui.ChapterTasks:Show()
		end
	elseif widget == "Collections_btn" then
		ui.Collections:Show()
	elseif widget == "Shop_btn" then
		ui.Bank:Show()
	elseif widget == "Options_btn" then
		ui.Options:Show()
	--[[elseif widget == "Friends_btn" then
		if facebook:IsLogged() then
			Facebook_AddToShow_InviteDialog(false)
		else
			gstore.action_after_facebook_login = 1
			if game_info.facebook.login_reward == false then
				ui.FBNeedLogin:Show()
			else
				facebook:LogIn()
			end
		end]]
	elseif widget == "FBConnect" then
		if not facebook:IsLogged() then
			if game_info.facebook.login_reward == false then
				ui.FBNeedLogin:Show()
			else
				facebook:LogIn()
			end
		end
	end
end

--------------------------------------------------------------------------------
--  TopPanel
--------------------------------------------------------------------------------
function OnDialogClick_TopPanel(dialog, widget)
	if widget == "BoardEnergyFill" or widget == "BoardEnergyFront" then
		ui.BuyEnergy:Show()
	elseif widget == "Gold" or widget == "Diamonds" then
		ui.Bank:Show()
	end
end

--------------------------------------------------------------------------------
--  Progress
--------------------------------------------------------------------------------
function OnDialogClick_Progress(dialog, widget)
	if widget == "Share" then
		local params = {
			p1 = string.format("%d%%", ui.Progress:GetDestProgress()),
			p2 = "_Common:" .. chapter:GetName()
		}
		facebook_posts.progress.picture = facebook_posts.progress.picture:gsub("progress[0-9]+", string.format("progress%03d", ui.Progress:GetDestProgress()))
		facebook:Share(facebook_posts.progress, params)
	elseif widget == "Cancel" and IsSharingEnabled() then
		kontagent:Event("cancelled", 0, "social", "posts", "case_progress")
	end
end

--------------------------------------------------------------------------------
function OnDialogClose_Progress(dialog)
	local cur_progress = ui.Progress:GetCurrentCaseProgress()
	if cur_progress == 100 then
		showCaseCompleteDialog = true
	end
	SafeCallFunction(OnNewProgress, cur_progress)
end

--------------------------------------------------------------------------------
--      ( )
--------------------------------------------------------------------------------
function OnDialogClick_LevelUp(dialog, widget)
	if widget == "Share" then
		local params = {
			p1 = game_info.xp_level + 1
		}
		facebook_posts.level_up.picture = facebook_posts.level_up.picture:gsub("level[0-9]+", string.format("level%02d", params.p1))
		facebook:Share(facebook_posts.level_up, params)
	elseif widget == "Cancel" or widget == "Ok" then
		if IsSharingEnabled() then
			kontagent:Event("cancelled", 0, "social", "posts", "level_up")
		end
		AddNewLevel()
		dialog:Close()
	end
end

--------------------------------------------------------------------------------
function AddNewLevel()
	kontagent:Event("level_up", -1, "player", "level_count", "")

	--  
	game_info.xp_level = game_info.xp_level + 1
	game_info.xp_points = game_info.xp_points - GetLevelXpPoints(game_info.xp_level)

	kontagent:Event("level_up", 1, "player", "level_count", "")
	kontagent:Event(string.format("level_%02d", game_info.xp_level), 0, "player", "level_funnel", "")

	--   
	if game_info.energy < new_player_settings.Energy then
		local energyToAdd = new_player_settings.Energy - game_info.energy
		game_info.energy_up_timer = energyToAdd * game_settings.AddEnergyPerTime
		kontagent:Event("level_up", energyToAdd, "economy", "energy", "source")
	end
end

--------------------------------------------------------------------------------
--   BuyEnergy
--------------------------------------------------------------------------------
function OnDialogClose_BuyEnergy(dialog)
	if IsEnergyLow() and facebook_settings.Enabled == 1 then
		--     Facebook,    
		if not facebook:IsLogged() and not game_info.facebook.login_reward then
			ui.FBNeedLogin:Show()
		--     
		--[[elseif facebook:IsLogged() == true then
			Facebook_AddToShow_InviteDialog(true)]]
		end
	end
end

--------------------------------------------------------------------------------
function IsEnergyLow()
	return (game_info.energy + (game_info.energy_up_timer / game_settings.AddEnergyPerTime) < GetPlayEnergyCost())
end

--------------------------------------------------------------------------------
function IsSharingEnabled()
	return (facebook:IsLogged() and facebook_settings.Share == 1)
end

--------------------------------------------------------------------------------
function ShowProgress()
	if not ui:IsModal() then
		local cur_progress = ui.Progress:GetCurrentCaseProgress()
		for k,v in pairs(chapter_progress) do
			if v.progress > cur_progress and ((primary_tasks:IsEmpty() and k == "FIN") or IsTaskExists(k)) then
				kontagent:Event("progress", 0, "case_progress", GetCaseNumber(), tostring(v.progress))
				if v.progress == 100 then
					kontagent:Event("complete", 0, "case_progress", GetCaseNumber(), "")
					--   
					local caseName = chapter:GetName()
					if not ui.TitleScreen:IsLastCase(caseName) then
						ui.TitleScreen:UnlockNewCase(caseName)
					end
					-- GameCenter achievement
					game_center:ReportAchievement(caseName .. "_completed")
				end

				local show_share = IsSharingEnabled()
				ui.Progress:SetWidgetVisible("Share",  show_share)
				ui.Progress:SetWidgetVisible("Ok", not show_share)
				ui.Progress:Show(v)
				return
			end
		end
	end
end

--------------------------------------------------------------------------------
function ShowCaseComplete()
	if not ui:IsModal() and showCaseCompleteDialog then
		--     
		ui.ModalDialogsShadow:SetFillColorAlpha(0)

		local caseName = chapter:GetName()
		if ui.TitleScreen:IsLastCase(caseName) then
			ui.CaseComplete:ShowTB(caseName)
		else
			ui.CaseComplete:Show()
		end

		showCaseCompleteDialog = nil
	end
end

--------------------------------------------------------------------------------
-- FACEBOOK
--------------------------------------------------------------------------------
function ProcessFacebookDialogs()
	if not facebook:IsLogged() then
		ui.DashboardPanel:SetWidgetVisible("FBConnect", (facebook_settings.Enabled == 1))
		return
	else
		ui.DashboardPanel:SetWidgetVisible("FBConnect", false)
	end

	if not ui:IsModal() then
		local dlg_switch = game_info.facebook.dlg_switch
		-- switch
		if dlg_switch == 0 then
			if Facebook_AddToShow_GetGiftDialog() == false then
				Facebook_AddToShow_SendGiftDialog(true)
			end
		elseif dlg_switch == 1 then
			if Facebook_AddToShow_SendGiftDialog(true) == false then
				Facebook_AddToShow_GetGiftDialog()
			end
		end

		dlg_switch = dlg_switch + 1
		if dlg_switch > 1 then
			dlg_switch = 0
		end
		game_info.facebook.dlg_switch = dlg_switch
	end
end

function OnGlobalStoreEvent_action_after_facebook_login(value)
	--[[if value == 2 then
		Facebook_AddToShow_InviteDialog(false)
	end]]
end

--[[function ValidateFriendsList()
	if CheckTimeForEvent(game_info.facebook.getfriend_time, facebook_settings.FriendsRefreshTime) then
		facebook:GetMeFriends()
		return false
	end
	return true
end]]

-- Show GETGIFT dialog with time control
function Facebook_AddToShow_GetGiftDialog()
	if CheckTimeForEvent(game_info.facebook.getgift_time, facebook_settings.GetGiftDialogTime) then
		game_info.facebook.getgift_show = false
		ui.FBGetGift:WaitResponce()
		facebook:GetMeRequests()
		return true
	end
	return false
end

-- Show SENDGIFT dialog
function Facebook_AddToShow_SendGiftDialog(with_time_control)
	if (with_time_control == false)
	or (CheckTimeForEvent(game_info.facebook.gift_time, facebook_settings.GiftDialogTime)) then
		ui.FBGift:SetMode("gift")
		ui.FBGift:SetGiftOptions("energy=2")
		ui.FBGift:Show()
		if with_time_control then
			game_info.facebook.gift_time = os.time()
		end
		if ui.FBGift:IsShown() then
			kontagent:Event("opened", 0, "social", "request", with_time_control and "auto" or "player")
			return true
		end
	end
	return false
end

-- Show INVITE dialog
function Facebook_AddToShow_InviteDialog(with_time_control)
	if (with_time_control == false)
	or (CheckTimeForEvent(game_info.facebook.invite_time, facebook_settings.InviteDialogTime)) then
		ui.FBGift:SetMode("invite")
		ui.FBGift:Show()
		if with_time_control then
			game_info.facebook.invite_time = os.time()
		end
		if ui.FBGift:IsShown() then
			kontagent:Event("opened", 0, "social", "invitation", with_time_control and "time" or "player")
		end
	end
end

--------------------------------------------------------------------------------
--  Facebook 
--------------------------------------------------------------------------------
function OnFacebookDialogClose(dlg_name, result)
	if dlg_name == "Dialog_Feed" then
		local st3 = ""
		if ui.LevelUp:IsShown() then
			st3 = "level_up"
			AddNewLevel()
			ui.LevelUp:Close()
		elseif ui.Progress:IsShown() then
			st3 = "case_progress"
			ui.Progress:Close()
		end
		kontagent:Event(result and "posted" or "cancelled", 0, "social", "posts", st3)
	end
end
