--------------------------------------------------------------------------------
--  File     : default_ispy_room.lua
--  Author(s): Pavel Kislyak, Pavel Demchuk
--
--  Summary  :  ISpy  " ".
--
--  Copyright  2013-2015 Hullabu Inc.
--------------------------------------------------------------------------------
dofile("default_room.lua")

DEBUG_TEST_POLYGONS = false

--------------------------------------------------------------------------------
function OnEntry()
	room.bonus:Clear()
	droppedBonusMap = {}
	completedFlag = false
	questItem = nil
	goldCoinsPicked = 0
	ispyTasksTotal = 0
	ispyTasksRemain = 0
	itemSearchTimer = 0

	--       ,   
	roomCard = dashboard:GetRoomCard(room:GetName())
	nextStar = math.min(roomCard:GetStarsCount() + 1, roomCard:GetMaxStarsCount())

	--  
	game_panel:EnableHintButton(true)
	game_panel:EnableMultiHintButton(true)

	-- ISpy 
	room.tasks:ClearAll()
	room.ispy:RestoreVisible()

	game_panel:SetModeHO()
	game_panel:SetTaskModeText()
	--game_panel:SetTaskModeAnagrams()
	--game_panel:SetTaskModeImages()
	--game_panel:SetTaskModeShapes()
	--room:SetPostProcessingFilter("black_white")
	--room:SetPostProcessingFilter("flashlight")
	--room:SetPostProcessingFilter("old_film")

	--   
	local shuffleDisabled = SafeCallFunction(OnAddCustomISpy)

	--   FindAll   
	if qe:IsFindAllMode() then
		room.ispy:RestoreUnusedList()
		room.ispy:AddTasksWithDefaultCount(nextStar - 1)
		ispyTasksTotal = room.ispy:GetDefaultTaskCount() + room.ispy:GetUnusedCount()
		ispyTasksRemain = room.ispy:GetUnusedCount()
	else
		--        
		local starTask = GetRoomStarTask(room:GetName(), nextStar)
		ispyTasksTotal = starTask.TasksCount or game_settings.TasksCount

		--        
		ispyTasksRemain = math.max(0, ispyTasksTotal - room.tasks:GetCount())
		--print(ispyTasksRemain)

		if game_settings.ISpyReusePercent and game_settings.ISpyReusePercent >= 0 and game_settings.ISpyReusePercent <= 100 then
			--    ,  ,    
			--           ,      6,       ISpyReusePercent
			local N = math.floor(ispyTasksRemain * game_settings.ISpyReusePercent / 100)
			ispyTasksRemain = ispyTasksRemain - room.ispy:AddUsedTasks(math.min(room.ispy:GetDefaultTaskCount() - room.tasks:GetCount(), N))
			--print(ispyTasksRemain)
		else
			print("game_settings.ISpyReusePercent value is invalid or nil!")
		end

		--   
		--    DEBUG_TEST_POLYGONS,        
		local N = (DEBUG_TEST_POLYGONS and room.ispy:GetCount() or math.min(room.ispy:GetDefaultTaskCount() - room.tasks:GetCount(), ispyTasksRemain))
		room.ispy:AddTasks(N, nextStar - 1)
		ispyTasksRemain = ispyTasksRemain - N
		--print(ispyTasksRemain)

		--  ,       
		if not shuffleDisabled then
			room.tasks:Shuffle()
		end
	end

	ui.GamePanel:SetISpyTaskInfo(0, ispyTasksTotal)

	SafeCallFunction(OnCustomEntry)

	if roomCard:IsDialogAssigned() then
		dialogOnEntry = ShowSpeakDialog(roomCard:GetAssignedDialog())
	end

	--       NPC
	if not dialogOnEntry then
		StartPlay()
	end

	kontagent:SetEventExtraParam("CurrentStar", tostring(nextStar))
	kontagent:Event("scene_start", 0, "scene_start", GetCaseNumber(), room:GetName())
end

--------------------------------------------------------------------------------
function OnUpdate(elapsedTime)
	if ui:IsModal() then
		return
	end

	if not ui.Tutorial:IsShown() then
		UpdateTimers(elapsedTime)
	end

	--       
	if not room.ispy:IsAllCompleted() and top_panel:IsRoomTimerShown() and top_panel.room_max_timer > 0 and top_panel.room_timer > 0 and top_panel.room_timer < game_settings.TimeLowWarningTime then
		if not room.sfx:IsPlaying("time_low") then
			room.sfx:PlayEx("time_low", true, 1.0, -0.5)
			ts:PlayLoop(ts.ALERT9, 0)
		end
		top_panel.room_timer_flash_alpha = (math.fmod(top_panel.room_timer, 0.4) > 0.2) and 0.8 or 0
	else
		top_panel.room_timer_flash_alpha = 0
		--    OnDialogShow,     -      ,   
	end

	--    Hint
	if itemSearchTimer > 20 then
		if tutorial:ShowWidget("GamePanel/ho_mode/btn_hint", "FreeHintButon", 1, -1, 720, 0, -1, true) then
			game_panel:EnableMultiHintButton(false)
			NO_DEC_HINT_COUNTER = 1
			game_info:CreateTextEvent("tutorial_step_08")
			kontagent:Event("tap_hint", 0, "tutorial", "step_08", "")
		end
		itemSearchTimer = 0
	end
end

--------------------------------------------------------------------------------
function UpdateTimers(elapsedTime)
	if not room.ispy:IsAllCompleted() and not chapter:IsInCloseup() then
		--   ( )
		if top_panel:IsRoomTimerShown() then
			--   (    )
			if top_panel.room_timer > 0 then
				top_panel.room_timer = top_panel.room_timer - elapsedTime
				if top_panel.room_timer <= 0 then
					top_panel.room_timer = 0
					ui.ModalDialogsShadow:SetFillColorAlpha(0.85)
					ui.TimeOut:Show(roomCard, game_settings.TimeOutExtraTime * game_settings.PlayTimeMultiplier, game_settings.TimeOutExtraTimeCost)
				end
			end
		end

		--     
		if top_panel:IsCupTimerShown() and top_panel.room_cup_timer > 0 then
			top_panel.room_cup_timer = top_panel.room_cup_timer - elapsedTime
			if top_panel.room_cup_timer < 0 then
				top_panel.room_cup_timer = 0
			end
		end

		--     
		itemSearchTimer = itemSearchTimer + elapsedTime
	end

	--  - (  ,   )
	if top_panel.room_bonus_timer > 0 then
		local mult_koef = (1 + math.ceil(top_panel.room_bonus_timer * game_settings.BonusTimerSegments / game_settings.BonusTimerMaxSec)) * 0.5
		top_panel.room_bonus_timer = top_panel.room_bonus_timer - elapsedTime * mult_koef
		if top_panel.room_bonus_timer < 0 then
			top_panel.room_bonus_timer = 0
		end
	end
end

--------------------------------------------------------------------------------
function OnDialogShow(dialog)
	if dialog:IsModal() and room.sfx:IsPlaying("time_low") then
		room.sfx:Stop("time_low")
		ts:Stop(ts.ALERT9)
	end
end

--------------------------------------------------------------------------------
function OnMouseClickOnBadItem(item)
	--  -   
	top_panel.room_bonus_timer = 0
	return false
end

--------------------------------------------------------------------------------
function OnPenaltyForBadClicks()
	if top_panel.room_timer <= 0 or room.tasks:IsAllCompleted() then
		return
	end

	penaltyIndex = penaltyIndex or 0
	if penaltyIndex < #game_settings.BadClickPenalties then
		penaltyIndex = penaltyIndex + 1
	end

	local penaltyTime = game_settings.BadClickPenalties[penaltyIndex]
	UpdateTimers(penaltyTime)

	room.sfx:Play("multiclick")
	room:FlyUpText(cursor:GetPosition(), string.format("-%02d:%02d", penaltyTime/60, penaltyTime%60), "time_icon")
	ts:Play(ts.TRIPLE_STRONG_CLICK_66, 0)
end

--------------------------------------------------------------------------------
function OnPickItem(item)
	--     
	itemSearchTimer = 0

	--  
	top_panel.room_score = top_panel.room_score + game_settings.ScoreOnPick * top_panel:GetCurrentScoreMult()
	local text = "+" .. game_settings.ScoreOnPick
	if top_panel:GetCurrentScoreMult() > 1 then
		text = text .. " x " .. top_panel:GetCurrentScoreMult()
	end
	room:FlyUpText(cursor:GetPosition(), text, '')

	--  -
	local segmentTime = game_settings.BonusTimerMaxSec / game_settings.BonusTimerSegments
	top_panel.room_bonus_timer = segmentTime * math.ceil((top_panel.room_bonus_timer + segmentTime) / segmentTime)
	if top_panel.room_bonus_timer > top_panel.room_max_bonus_timer then
		top_panel.room_bonus_timer = game_settings.BonusTimerMaxSec
	end

	--  
	--      
	--[[local dt = os.date("*t")
	if dt.wday == 1 and dt.yday ~= game_info.golden_rain_last_day and (remainNewISpyTasks + room.tasks:GetCount() - room.tasks:GetCompletedCount()) == 2 then
		game_info.golden_rain_last_day = dt.yday
		local averagePlayerClicksPerSecond = 6
		local coinsToDrop = math.min(math.floor(top_panel.room_timer * averagePlayerClicksPerSecond), 200)
		for i=1,coinsToDrop do
			room.bonus:Drop("gold")
		end
	else]]
		local count = 0
		local bonusProfile = GetBonusProfile(room:GetName())
		count = count + DropBonusByCompletedTasks("diamonds", bonusProfile.diamonds)
		count = count + DropBonusByCompletedTasks("gold", bonusProfile.gold)
		count = count + DropBonusByCompletedTasks("energy", bonusProfile.energy)

		if count > 0 then
			ts:Play(ts.FAST_PULSE_33, 0.2)
		end
	--end

	--   
	if room.ispy:IsAllCompleted() and ispyTasksRemain <= 0 then
		top_panel:SetEnabledExitBtn(false)
	end

	local itemsFound = room.tasks:GetCompletedCount() + (ispyTasksTotal - ispyTasksRemain - room.tasks:GetCount())
	ui.GamePanel:SetISpyTaskInfo(itemsFound, ispyTasksTotal)
end

--------------------------------------------------------------------------------
function OnEndPickedItemAnim(item)
	if ispyTasksRemain > 0 then
		if room.tasks:GetCompletedCount() >= game_settings.CompletedTasksReplaceFreq then
			--   
			for i = 0,room.tasks:GetCount()-1 do
				local task = room.tasks:GetTaskByNum(i)
				if task:IsCompleted() and task:GetTag() == "ISpy" then
					if room.ispy:ReplaceTask(task:GetName(), nextStar - 1) then
						ispyTasksRemain = ispyTasksRemain - 1
						if ispyTasksRemain == 0 then break end
					end
				end
			end
			hints:CreateFindList()
		end
	elseif room.ispy:IsAllCompleted() and not completedFlag then
		top_panel.room_bonus_timer = 0
		top_panel:SetEnabledExitBtn(true)
		OnISpyComplete()
	end
end

--------------------------------------------------------------------------------
--     ISpy ( ,      )
--------------------------------------------------------------------------------
function OnISpyComplete()
	CompleteRoom()
end

--------------------------------------------------------------------------------
--          
--------------------------------------------------------------------------------
function DropBonusByCompletedTasks(bonusName, maxDropsPerPlay)
	if type(maxDropsPerPlay) ~= 'number' then
		return 0
	end

	--      
	local droppedCount = (droppedBonusMap[bonusName] or 0)
	local prev = droppedCount

	local limit
	if maxDropsPerPlay < 0 then
		--   maxDropsPerPlay ,        
		limit = -maxDropsPerPlay + droppedCount
	else
		--  ,         (   )
		limit = math.min(maxDropsPerPlay, math.abs(maxDropsPerPlay / (room.tasks:GetCount() - 1) * room.tasks:GetCompletedCount()))
	end

	while droppedCount < limit do
		room.bonus:Drop(bonusName)
		droppedCount = droppedCount + 1
	end

	-- C    
	droppedBonusMap[bonusName] = droppedCount

	--  ,    
	return (droppedCount - prev)
end

--------------------------------------------------------------------------------
--   
--------------------------------------------------------------------------------
function OnPickBonus(bonusName, isTapped, tapPosition)
	if isTapped then
		room.sfx:Play("bonus_pick")
	end
	if bonusName == "gold" then
		goldCoinsPicked = goldCoinsPicked + 1
		room:FlyUpText(tapPosition, '+1', '')
	end
end

--------------------------------------------------------------------------------
function StartPlay()
	--    top_panel
	top_panel.room_score = 0
	top_panel.room_bonus_timer = 0
	top_panel.room_max_bonus_timer = game_settings.BonusTimerMaxSec
	top_panel:ShowRoomMode(true)
	top_panel:SetEnabledExitBtn(true)

	--  
	local playTime = qe:IsFindAllMode() and 0 or GetRoomPlayTime(room:GetName(), roomCard:GetStarsCount() + 1)
	if playTime > 0 and ui.StartRoom.boost_1 then
		playTime = playTime + game_settings.BoostExtraTimeValue
	end
	top_panel:StartRoomTimer(playTime)
	top_panel.room_timer_flash_alpha = 0

	--      
	top_panel:SetVisibleCupTimer(false)
	if playTime > 0 and not roomCard:IsWinCup() then
		top_panel:StartCupTimer(GetRoomGoldCupTime(room:GetName()))
	end
end

--------------------------------------------------------------------------------
--   
--------------------------------------------------------------------------------
function OnHideActiveTutroial(id, aw_name)
	if id == "FreeHintButon" then
		game_panel:EnableMultiHintButton(true)
		if aw_name ~= "btn_hint" then
			NO_DEC_HINT_COUNTER = nil
		end
	end
end

--------------------------------------------------------------------------------
function CompleteRoom(param)
	completedFlag = true

	--  param - ,    ,    
	if type(param) == 'number' then
		--    
		local a = GetRoomStarTask(room:GetName(), math.floor(param))
		local b = GetRoomStarTask(room:GetName(), math.floor(param + 1)) or a
		if a and b then
			minForcedScore = math.floor(a.Score + (b.Score - a.Score) * math.fmod(param, 1))
		end
		param = nil
	end

	if param then
		--   
		room.bonus:PickAll()

		--  
		game_info.gold = game_info.gold + goldCoinsPicked

		--     
		local winGoldCup = false
		if not roomCard:IsWinCup() then
			if top_panel.room_max_timer > 0 and (top_panel.room_max_timer - top_panel.room_timer) < GetRoomGoldCupTime(room:GetName()) then
				roomCard:WinCup()
				winGoldCup = true
				--[[if cstore.GoldCupWin == nil then cstore.GoldCupWin = 1
				else cstore.GoldCupWin = cstore.GoldCupWin + 1
				end]]
			end
		end

		--  Kontagent -     , ..      OnDialogClose_RoomComplete()
		ktgCollectibleAssigned = roomCard:IsCollectibleAssigned()
		-- C      
		ktgStarsProgress = roomCard:GetStarsCount() + roomCard:GetNextStarProgress()
		--  ,   
		ktgBoostScoreMult = ui.StartRoom.boost_3 and game_settings.BoostScoreMultiplierValue or 1

		--    
		ui.RoomComplete:Show(roomCard, winGoldCup, ktgBoostScoreMult, minForcedScore or 0)
	else
		if roomCard:IsItemAssigned() and roomCard:GetStarsCount() >= roomCard:GetAssignedItemNeedStars() then
			questItem = roomCard:GetAssignedItem()
			roomCard:ClearAssignedItem()

			--  
			if chapter.tasks:Exists(questItem) then
				chapter.tasks:GetTask(questItem):MarkCompleted()
			end

			--       ,   
			if quest_items:IsInvItemExist(split(questItem, ',')[1]) then
				pick_panel:AddQuestItem(questItem)
				return
			--  ,        
			elseif ShowSpeakDialog(questItem, questItem .. "_IMAGE") then
				return
			end
		end

		--  ,           ,     RoomComplete
		CompleteRoom(true)
	end
end

--------------------------------------------------------------------------------
--   RoomComplete
--------------------------------------------------------------------------------
function OnDialogClose_RoomComplete(dialog)
	local caseNum	= GetCaseNumber()
	local roomName	= room:GetName()
	local baseScore	= ktgBoostScoreMult * top_panel.room_score
	local timeScore	= ktgBoostScoreMult * CalcRoomBonusTimeScore(roomCard)

	kontagent:Event("scene_end", roomCard:GetStarsCount() - math.floor(ktgStarsProgress), "economy", "stars", "source")
	kontagent:Event("scene_end", goldCoinsPicked, "economy", "gold", "source")
	kontagent:Event("score", baseScore, "scene_end", caseNum, roomName)
	kontagent:Event("time_bonus", timeScore, "scene_end", caseNum, roomName)
	kontagent:Event("final_score", baseScore + timeScore, "scene_end", caseNum, roomName)
	kontagent:SetEventExtraParam("CurrentStar", tostring(nextStar))
	kontagent:Event("percent_of_star", math.floor((roomCard:GetStarsCount() + roomCard:GetNextStarProgress() - ktgStarsProgress) * 100), "scene_end", caseNum, roomName)
	kontagent:Event("time_to_complete", top_panel.room_max_timer - top_panel.room_timer, "scene_end", caseNum, roomName)

	if ktgCollectibleAssigned then
		kontagent:Event("painting_fragment", 0, "scene_end", caseNum, roomName)
	end

	JumpDashboard()

	if not primary_tasks:IsEmpty() then
		CalcCollectibles()
	end
end

--------------------------------------------------------------------------------
--     
--------------------------------------------------------------------------------
function OnDialogClose_PickItems(dialog)
	if not questItem or not ShowSpeakDialog(questItem) then
		CompleteRoom(true)
	end
end

--------------------------------------------------------------------------------
--    PersSpeak
--------------------------------------------------------------------------------
function OnDialogClose_PersSpeak(dialog)
	if dialogOnEntry then
		StartPlay()
		dialogOnEntry = nil
	elseif completedFlag then
		CompleteRoom(true)
	end

	SafeCallFunction(OnCloseSpeakDialog, dialog:GetUserTag())
end

--------------------------------------------------------------------------------
--   TimeOut
--------------------------------------------------------------------------------
function OnDialogClick_TimeOut(dialog, widget)
	if widget == "Cancel" or widget == "OkBtn" then
		kontagent:SetEventExtraParam("CurrentStar", tostring(nextStar))
		kontagent:Event("scene_failed", top_panel.room_max_timer, "scene_end", GetCaseNumber(), room:GetName())
		JumpDashboard()
	elseif widget == "BuyExtraTime" then
		if game_info.diamonds >= game_settings.TimeOutExtraTimeCost then
			game_info.diamonds = game_info.diamonds - game_settings.TimeOutExtraTimeCost
			top_panel.room_timer = game_settings.TimeOutExtraTime * game_settings.PlayTimeMultiplier
			top_panel.room_max_timer = top_panel.room_max_timer + top_panel.room_timer
			game_info:CreateGameBuyEvent("extra_time", Currency_Diamonds, game_settings.TimeOutExtraTimeCost, room:GetName())
			dialog:Close()
		else
			ui.Bank:Show()
			ui.Bank:ShowNotEnoughBalloon(Currency_Diamonds, game_settings.TimeOutExtraTimeCost - game_info.diamonds)
			ui.Bank:SetBuyReason("TIME_OVER")
		end
	end
end

--------------------------------------------------------------------------------
function OnFocusLost()
	if completedFlag == false then
		ui:ShowDialog("Pause")
	end
end

--------------------------------------------------------------------------------
function OnStartHint(count)
	kontagent:SetEventExtraParam("Hints_Left", tostring(count == 1 and game_info.hints - 1 or game_info.multi_hints - 1))
	kontagent:SetEventExtraParam("Item_Hinted", hints:GetLastHintedItems(count))
	kontagent:Event(count == 1 and "single_hint" or "triple_hint", math.floor(itemSearchTimer), "player", GetCaseNumber(), room:GetName())
end

--------------------------------------------------------------------------------
--     ,      ,  
--------------------------------------------------------------------------------
function AddOrHideExcludedItem(itemName, initiallyVisible)
	local keyName = room:GetName() .. "_" .. itemName
	if roomCard:IsItemAssigned() and set(split(roomCard:GetAssignedItem(), ","))[itemName] then
		room.ispy:AddTaskForRequiredItem(itemName)
		cstore:SetKey(keyName, true)
		return true
	elseif not initiallyVisible or cstore:IsKeyExists(keyName) then
		if room:IsItemExisting(itemName) then
			--   
			room:GetItem(itemName):Hide()
		else
			--   ,      
			local count = room.im:GetItemCount()
			for i = 0,count-1 do
				local item = room.im:GetItemByNum(i)
				if item:GetISpyGroup() == itemName then
					item:Hide()
				end
			end
		end
	end
end
