--------------------------------------------------------------------------------
--  File     : qe_script.lua
--  Author(s): Sergey Yakutin, Pavel Kislyak, Pavel Demchuk
--
--  Summary  :    QuestEngine.
--
--  Copyright  2013-2015 Hullabu Inc.
--------------------------------------------------------------------------------
dofile("_Chapters/_common_lua/_config.lua")
dofile("_Chapters/_common_lua/_helpers.lua")

START_CASE_NAME			= "case00"
roomCommonResGroup 		= "RoomCommon"
load_common_res_delay	= 0
send_session_info_flag	= true
cloud_conflict_flag		= true
restore_chapter_flag	= false

-- states for YesNo dialog
DLG_STATE_GAME_EXIT		= "game_exit"
DLG_STATE_DASHEXIT		= "dashboard_exit"
DLG_STATE_ROOMEXIT		= "room_exit"
DLG_STATE_FIRST_UPD		= "first_upd"

ENABLE_SERIALIZATION	= true

--------------------------------------------------------------------------------
--  
--------------------------------------------------------------------------------
--  GUI 
function Create_GUI_LoadTask(folder_name)
	folder_prefix = "gui:" .. folder_name .. "/"

	resource_dir = folder_prefix .. "Images"
	if resm:IsDirExists(resource_dir) then
		resm:AddSpriteDirToTask(resource_dir, folder_name)
		resm:AddMaskDirToTask  (resource_dir, folder_name)
	end

	resource_dir = folder_prefix .. "ImagesExt"
	if resm:IsDirExists(resource_dir) then
		resm:AddSpriteDirToTask(resource_dir, folder_name)
	end

	resource_dir = folder_prefix .. "Locale"
	if resm:IsDirExists(resource_dir) then
		resm:AddStringDirToTask(resource_dir, folder_name)
	end

	resource_dir = folder_prefix .. "Anim"
	if resm:IsDirExists(resource_dir) then
		resm:AddAnimDirToTask  (resource_dir, folder_name)
	end

	resource_dir = folder_prefix .. "ParticleGroups"
	if resm:IsDirExists(resource_dir) then
		resm:AddParticleGroupDirToTask(resource_dir, folder_name)
	end
end

--   YesNo
function Show_YN_Dialog(caption, tag)
	ui.DialogYesNo:SetUserTag(tag)
	ui.DialogYesNo:SetCaption(caption)
	ui.DialogYesNo:Show()
end

--------------------------------------------------------------------------------
--    (   !)
--------------------------------------------------------------------------------
function InitNewUserInfo()
	game_info.energy					= new_player_settings.Energy
	game_info.energy_bonus				= 0
	game_info.energy_update_time		= os.time()
	game_info.energy_up_timer			= 0
	game_info.gold						= new_player_settings.Gold
	game_info.diamonds					= new_player_settings.Diamonds
	game_info.hints						= new_player_settings.Hints
	game_info.multi_hints				= new_player_settings.MultiHints
	game_info.xp_points					= 0
	game_info.xp_level					= 1
	game_info.intro_shown				= false
	game_info.case_progress.case00		= 0

	-- FACEBOOK
	game_info.facebook.invite_time		= 0
	game_info.facebook.login_reward		= false
	game_info.facebook.getfriend_time	= 0
	game_info.facebook.getgift_time		= 0
	game_info.facebook.getgift_show		= false
	game_info.facebook.gift_time		= 0
	game_info.facebook.gift_show		= false
	game_info.facebook.dlg_switch		= 0
	facebook:ClearProfile()

	-- GAMECENTER
	game_info.game_center.auto_login	= true
	game_info.game_center.id			= ""
	game_info.game_center.alias			= ""

	-- Collections
	game_info.paintings_assembled		= 0

	-- Kontagent
	game_info.session_number			= 0
	game_info.iap_count					= 0

	LogInfo("New user profile created!")
end

--------------------------------------------------------------------------------
--       (   !)
--------------------------------------------------------------------------------
function ImportGlobalUserInfo(importVersion)
	--   Json
	if not game_info.xp_level then
		game_info.facebook.login_reward	= false
		game_info.xp_points				= 0
		game_info.xp_level				= 1
	end

	--  hints -> multi_hints
	if not game_info.multi_hints then
		game_info.multi_hints			= game_info.hints
		game_info.hints					= new_player_settings.Hints
	end

	if not game_info.facebook.invite_time then
		game_info.facebook.invite_time	= 0
	end

	game_info.facebook.getfriend_time	= 0
	game_info.facebook.getgift_time		= 0
	game_info.facebook.getgift_show		= false
	game_info.facebook.gift_time		= 0
	game_info.facebook.gift_show		= false
	game_info.facebook.dlg_switch		= 0

	if not game_info.energy_bonus then
		game_info.energy_bonus			= 0
	end

	-- Collections
	if not game_info.paintings_assembled then
		game_info.paintings_assembled	= 0
	end

	-- Kontagent
	if not game_info.session_number then
		game_info.session_number		= 0
		game_info.iap_count				= 0
	end

	LogInfo("Imported old user profile!")
end

--------------------------------------------------------------------------------
--   - Let's go
--------------------------------------------------------------------------------
function GameStart(name, enable_serialization)
	if qe:PlayGame(name .. ".list", "GameChapter", not enable_serialization) then
		lastStory = name
	elseif qe:GetStartGameResult() == qe.START_BadSave then
		ui.DialogOk:SetCaption("_Common:bad_saves")
		ui.DialogOk:SetUserTag("bad_qe_save")
		ui.DialogOk:Show()
	end
end

--------------------------------------------------------------------------------
--   
--------------------------------------------------------------------------------
function OnIncompatibleSavedFile()
	ui.DialogOk:SetCaption("_Common:saves_incompatible")
	ui.DialogOk:SetUserTag("incomp_chapter_save")
	ui.DialogOk:Show()
end

--------------------------------------------------------------------------------
function OnStartEngine()
	--    
	resm:AddSoundDirToTask("sounds:Preload", "")
	resm:AddFontDirToTask("fonts:Preload", "")
	Create_GUI_LoadTask("_Common")
	Create_GUI_LoadTask("CheckUpdates")
	Create_GUI_LoadTask("TitleScreen")
	resm:LoadTasksAtOnce()
	ui:LoadAndCreateDialogs("gui:_templates.agui")
	ui:LoadAndCreateDialogs("gui:AlertDialog.agui")
	ui:LoadAndCreateDialogs("gui:CheckUpdates.agui")
	ui:LoadAndCreateDialogs("gui:LoadingScreen.agui")
	ui:LoadAndCreateDialogs("gui:TitleScreen.agui")
end

--------------------------------------------------------------------------------
function OnGameDataReady()
	--   
	ui.CheckUpdates:Show(true)
	--       
	ui.LoadingScreen:Show()
	--   ,    
	if ui.CheckUpdates:IsShown() then
		ui.CheckUpdates:SetUserTag(DLG_STATE_FIRST_UPD)
	else
		load_common_res_delay = 2
	end
end

--------------------------------------------------------------------------------
--   
--------------------------------------------------------------------------------
function OnDialogClose_CheckUpdates(dialog)
	--       (  )
	if dialog:GetUserTag() == DLG_STATE_FIRST_UPD then
		dialog:SetUserTag("")
		load_common_res_delay = 2
	end
end

--------------------------------------------------------------------------------
function LoadCommonRes()
	--      
	resm:AddSoundDirToTask			("sounds:",				"")
	resm:AddSoundDirToTask			("sounds:Room",			roomCommonResGroup)
	resm:AddSpriteDirToTask			("particles:Images",	"~Particles~")
	resm:AddParticleDirToTask		("particles:",			"")
	resm:AddParticleGroupDirToTask	("particles:",			"")
	resm:AddFontDirToTask 			("gamedata:Fonts",		"")
	--resm:AddAnimDirToTask			("gamedata:Anim",		"")

	--     UI
	Create_GUI_LoadTask("_CommonAtlases")
	Create_GUI_LoadTask("Options")
	Create_GUI_LoadTask("PersComment")
	Create_GUI_LoadTask("PersSpeak")
	Create_GUI_LoadTask("TopPanel")
	Create_GUI_LoadTask("DashboardPanel")
	Create_GUI_LoadTask("GamePanel")
	Create_GUI_LoadTask("PickItems")
	Create_GUI_LoadTask("ChapterTasks")
	Create_GUI_LoadTask("StartRoom")
	Create_GUI_LoadTask("RoomComplete")
	Create_GUI_LoadTask("Bank")
	Create_GUI_LoadTask("BuyEnergy")
	Create_GUI_LoadTask("BuyHint")
	Create_GUI_LoadTask("BuyMultiHint")
	Create_GUI_LoadTask("TimeOut")
	Create_GUI_LoadTask("Pause")
	Create_GUI_LoadTask("Tutorial")
	Create_GUI_LoadTask("CaseComplete")
	Create_GUI_LoadTask("MiniGameFailed")
	Create_GUI_LoadTask("FBGetGift")
	Create_GUI_LoadTask("FBGift")
	Create_GUI_LoadTask("FBNeedLogin")
	Create_GUI_LoadTask("LevelUp")
	Create_GUI_LoadTask("NotEnoughStars")
	Create_GUI_LoadTask("HintMG")
	Create_GUI_LoadTask("RoomUnlocked")
	Create_GUI_LoadTask("UnlockRoom")
	Create_GUI_LoadTask("Progress")
	Create_GUI_LoadTask("WeAreSorry")
	Create_GUI_LoadTask("DailyReward")
	Create_GUI_LoadTask("Collections")
	Create_GUI_LoadTask("PaintView")
	Create_GUI_LoadTask("CloudConflict")

	qe:StartLoading()
end

--------------------------------------------------------------------------------
function OnExitEngine()
end

--------------------------------------------------------------------------------
--     ( qe:StartLoading())
--------------------------------------------------------------------------------
function OnStartLoadResources()
	playhaven:InitPrice(prices.Playhaven)
	playhaven:PreloadAdContent("case_selection_screen")
end

--------------------------------------------------------------------------------
function OnEndLoadResources()
	ui:LoadAndCreateDialogs("gui:_index.agui")
	ui:CloseDialog("LoadingScreen")

	ui.Bank:InitSpecActionPrice(prices.SpecialOffers)
	ui.Bank:InitGoldPrice(prices.Gold)
	ui.Bank:InitDiamondsPrice(prices.Diamonds)
	ui.BuyEnergy:InitPrice(prices.Energy)
	ui.BuyHint:InitPrice(prices.Hints)
	ui.BuyMultiHint:InitPrice(prices.MultiHints)
	ui.FBNeedLogin:InitPrice(rewards.facebook_login)

	--  Facebook ,    
	if facebook_settings.Enabled == 1 then
		gstore.action_after_facebook_login = -1
		facebook:AutoLogIn()
	end

	--   
	kontagent:Event(string.format("level_%02d", game_info.xp_level), 0, "player", "level_funnel", "")

	--   
	UpdateDailyReward()

	--   
	if gameini:IsNeedUnlockCases() then
		for i = 1,gameini:GetLastCase() do
			local caseName = string.format("case%02d", i)
			game_info.case_progress[caseName] = game_info.case_progress[caseName] or 0
			if ui.TitleScreen:IsLastCase(caseName) then
				break
			end
		end
	end

	if gameini:IsFastStart() then
		OnClickOnCase(START_CASE_NAME)
	else
		--    
		ShowCaseScreen()
	end
end

--------------------------------------------------------------------------------
--   
--------------------------------------------------------------------------------
function ShowCaseScreen()
	if game_info.intro_shown then
		--  ,    
		music:Stop(true)
		ambient:Play("case_screen")

		--      
		ui.TitleScreen:SetWidgetVisible("FBConnect", not facebook:IsLogged() and not facebook:IsAuthRequestsExits() and facebook_settings.Enabled == 1)
		ui.TitleScreen:SetWidgetVisible("GooglePlay", GOOGLE or false)
		ui.TitleScreen:SetWidgetVisible("Exit", (GOOGLE or AMAZON) or false)
		ui.TitleScreen:Show()
	else
		--  -
		ShowVideo("intro", 0, 0, "show case screen")
		game_info.intro_shown = true
		game_info:CreateTextEvent("tutorial_step_02")
		kontagent:Event("cut_scene", 0, "tutorial", "step_02", "")
	end
end

--------------------------------------------------------------------------------
--  -
--------------------------------------------------------------------------------
function OnDialogClose_VideoPlayer(dialog)
	if dialog:GetUserTag() == "show case screen" then
		ShowCaseScreen()
	end
end

--------------------------------------------------------------------------------
function OnDialogShow_TitleScreen(dialog)
	--     ,      
	local timeToAds = (game_info.case_progress.case00 == 100 and game_info.case_progress.case01 ~= 0)
	SHOW_GPN_AD = timeToAds and game_info.iap_count == 0
	SHOW_PLH_AD = timeToAds

	--  AdX-
	local referral = adx:GetReferral()
	if referral ~= "" then
		--  
		if not game_info.adx_referral_sent_to_kontagent then
			kontagent:Event("referral", 0, "hidden_artifacts", "adx", referral)
			game_info.adx_referral_sent_to_kontagent = true
		end
		--    
		if not game_info.adx_referral_sent_to_pawlostat then
			game_info:CreateTextEvent("referral:" .. referral)
			game_info.adx_referral_sent_to_pawlostat = true
		end
	end

	if cloud_conflict_flag then
		cloud_conflict_flag = false
		if game_center:CloudGetState() ~= 0 then
			ui.CloudConflict:Show()
			return
		end
	end

	--  
	if send_session_info_flag then
		qe:SendSessionInfo()
		send_session_info_flag = false
	end

	--     ,     
	if game_info.case_progress.case00 == 100 and game_info.case_progress.case01 == 0 then
		tutorial:ShowWidget("TitleScreen/case01", "NewCase", 1, -1, -1, 0, -1, false)
		return
	end

	--  ,   -     
	if tutorial:ShowWidget("TitleScreen/case00", "FirstCase", 1, -1, -1, 0, -1, false) then
		game_info:CreateTextEvent("tutorial_step_03")
		kontagent:Event("tap_first_case", 0, "tutorial", "step_03", "")
	end
end

--------------------------------------------------------------------------------
-- TITLE SCREEN
--------------------------------------------------------------------------------
function OnDialogClick_TitleScreen(dialog, widget)
	if widget == "FBConnect" then
		if not facebook:IsLogged() then
			if game_info.facebook.login_reward == false then
				ui.FBNeedLogin:Show()
			else
				facebook:LogIn()
			end
		end
	elseif widget == "GooglePlay" then
		game_center:ShowAchievements()
	elseif widget == "Exit" then
		Show_YN_Dialog("_Common:EXIT_GAME", DLG_STATE_GAME_EXIT)
	end
end

--------------------------------------------------------------------------------
--  
--------------------------------------------------------------------------------
function OnFocusGain()
	--   
	UpdateDailyReward()
end

--------------------------------------------------------------------------------
function OnUpdate(elapsedTime)
	--         ,   LoadingScreen    
	if load_common_res_delay > 0 then
		load_common_res_delay = load_common_res_delay - 1
		if load_common_res_delay == 0 then
			LoadCommonRes()
		end
		return
	end

	--    
	if not ui:IsModal() and (ui.TitleScreen:IsShown() or ui.DashboardPanel:IsShown()) then
		ShowDailyReward()
	end

	--   GPN/Playhaven
	if not ui:IsModal() and ui.TitleScreen:IsShown() then
		if not playhaven:IsAdShown() and SHOW_GPN_AD and gpn:ShowAd() then
			SHOW_GPN_AD = false
		elseif not gpn:IsAdShown() and SHOW_PLH_AD then
			playhaven:ShowAd("case_selection_screen")
			SHOW_PLH_AD = false
		end
	end
end

--------------------------------------------------------------------------------
function OnStartChapter()
end

--------------------------------------------------------------------------------
function OnGameOver()
end

--------------------------------------------------------------------------------
function OnNextChapter()
	-- - 
	qe:SaveGameToProfile()
end

--------------------------------------------------------------------------------
function OnExitGame()
	if not restore_chapter_flag then
		ui.DashboardPanel:Close()
		ui.ChapterTasks:Close()
		ui.GamePanel:Close()
		ui.StartRoom:Close()
		ui.TopPanel:Close()

		ShowCaseScreen()
	end
end

--------------------------------------------------------------------------------
--   YesNo
--------------------------------------------------------------------------------
function OnDialogClick_DialogYesNo(dialog, widget)
	if widget == "Yes" then
		local state = dialog:GetUserTag()
		if state == DLG_STATE_DASHEXIT then
			dialog:Close()
			dialog:SetUserTag("")
			qe:SaveGameToProfile()
			qe:ExitGame()
		elseif state == DLG_STATE_ROOMEXIT then
			dialog:Close()
			dialog:SetUserTag("")
			restore_chapter_flag = true
			qe:ExitGame()
			GameStart(lastStory, ENABLE_SERIALIZATION)
			restore_chapter_flag = false
		elseif state == DLG_STATE_GAME_EXIT then
			qe:ExitEngine()
		end
	elseif widget == "No" then
		dialog:Close()
	end
end

--------------------------------------------------------------------------------
--   MiniGameFailed
--------------------------------------------------------------------------------
function OnDialogClick_MiniGameFailed(dialog, widget)
	if widget == "Ok" then
		dialog:SetUserTag("")
		dialog:Close()
		restore_chapter_flag = true
		qe:ExitGame()
		GameStart(lastStory, ENABLE_SERIALIZATION)
		restore_chapter_flag = false
	end
end

--------------------------------------------------------------------------------
--  PAUSE
--------------------------------------------------------------------------------
function OnDialogClick_Pause(dialog, widget)
	dialog:Close()
end

--------------------------------------------------------------------------------
--  DialogOk  
--------------------------------------------------------------------------------
function OnDialogClick_DialogOk(dialog, widget)
	if widget == "Ok" then
		dialog:Close()
		local state = dialog:GetUserTag()
		dialog:SetUserTag("")
		if state == "bad_qe_save" then
			GameStart(START_CASE_NAME, false)
		elseif state == "incomp_chapter_save" then
			qe:RestartCurrentChapter()
		end
	end
end

--------------------------------------------------------------------------------
-- CASE SELECTOR
--------------------------------------------------------------------------------
function OnClickOnCase(name)
	--   Playhaven
	playhaven:PreloadAdContent("case_" .. GetCaseNumber(name) .. "_scene_selection_screen")
	playhaven:PreloadAdContent("bank_open")
	playhaven:PreloadAdContent("case_selection_screen")

	ambient:Stop()
	ui:CloseDialog("TitleScreen")
	continue_flag = ENABLE_SERIALIZATION
	GameStart(name, continue_flag)
end

--------------------------------------------------------------------------------
-- DASHBOARD
--------------------------------------------------------------------------------
function OnDialogClick_TopPanel(dialog, widget)
	if widget == "DashboardExit" then
		Show_YN_Dialog("_Common:EXIT_TO_CASE_SCREEN", DLG_STATE_DASHEXIT)
	elseif widget == "HomeButton" then
		ui.ModalDialogsShadow:SetFillColorAlpha(0.85)
		Show_YN_Dialog("_Common:EXIT_TO_DASHBOARD", DLG_STATE_ROOMEXIT)
	elseif widget == "OptionsButton" then
		ui.ModalDialogsShadow:SetFillColorAlpha(0.85)
		ui.Options:Show()
	end
end

--------------------------------------------------------------------------------
-- CASE COMPLETE
--------------------------------------------------------------------------------
function OnDialogClose_CaseComplete(dialog)
	if dialog:IsLastCase() then
		print("Last CASE!")
	end

	qe:SaveGameToProfile()
	qe:ExitGame()

	--     case00     
	if game_info.case_progress.case01 == 100 then
		ShowRateDialog(0.3)
	end
end

--------------------------------------------------------------------------------
--   Rate Now
--------------------------------------------------------------------------------
function OnGlobalStoreEvent_RATE_NOW(value)
	if not game_info.version_rated and game_settings.RatingURI and game_settings.AppStoreID then
		--          0.5 , GPN-     softlock
		ui.AlertDialog:Show(1, "_Common:RATE_CAPTION", "_Common:RATE_TEXT", game_settings.AppStoreID, "_Common:RATE_BTN_LATER", "_Common:RATE_BTN_RATE", "", value/1000, 0.5)
	end
end

function OnCloseAlertDialog(id, res)
	if id == 1 then
		--   rate now
		if res == 1 and OpenUrl(game_settings.RatingURI) then
			game_info.version_rated = qe:GetVersion()
		end
		kontagent:Event((res == 1) and "rated" or "closed", 0, "dialog", "action", "rate_game")
	end
end

--------------------------------------------------------------------------------
-- FACEBOOK
--------------------------------------------------------------------------------
--      ,      Chapter'e
function OnFacebookLogin(state)
	if state == true then
		if game_info.facebook.login_reward == false then
			--   
			LogInfo("*FB* Got login reward in qe_script!")
			TransferRewardToUser(rewards.facebook_login, "fb_connect")
			game_info.facebook.login_reward = true
		end
		ui.FBNeedLogin:Close()
		ui.TitleScreen:SetWidgetVisible("FBConnect", false)
	else
		ui.TitleScreen:SetWidgetVisible("FBConnect", true)
		if gstore.action_after_facebook_login ~= -1 then
			ui.AlertDialog:Show(0, "FBNeedLogin:ALERT_CAPTION", "FBNeedLogin:ALERT_TEXT", "", "_Common:btn_ok", "", "", 0.4, 0)
		end
	end
	gstore.action_after_facebook_login = 0
end

--------------------------------------------------------------------------------
--  FBNeedLogin
--------------------------------------------------------------------------------
function OnDialogClick_FBNeedLogin(dialog, widget)
	if widget == "fb_connect" then
		facebook:LogIn()
	elseif widget == "fb_close" then
		gstore.action_after_facebook_login = 0
		dialog:Close()
	end
end
